@@ -25,13 +25,11 @@
 import base64
 import binascii
 import re
-try:
-    set
-except NameError:
-    from sets import Set as set
 
 from repoze.workflow.statemachine import StateMachine, StateMachineError
 
+from pymta.compat import set
+from pymta.exceptions import InvalidParametersError, SMTPViolationError
 from pymta.model import Message, Peer
 
 
@@ -43,26 +41,7 @@ __all__ = ['SMTPSession']
 regex_string = r'^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$'
 
 
-class PyMTAException(Exception):
-    def __init__(self, response_sent=False, code=550, 
-                 reply_text='Administrative Prohibition'):
-        self.response_sent = response_sent
-        self.code = code
-        self.reply_text = reply_text
-
-class DynamicStateSwitchException(PyMTAException):
-    """Used to implement more complicated state transitions where the next 
-    state depends on other values (e.g. where to go after a message was 
-    received - depends on ESMTP and AUTH)."""
-    def __init__(self, new_state):
-        self.new_state = new_state
-
-class InvalidParametersException(PyMTAException):
-    def __init__(self, parameter=None, *args, **kwargs):
-        self.parameter = parameter
-        super(InvalidParametersException, self).__init__(*args, **kwargs)
-
-class PolicyDenial(PyMTAException):
+class PolicyDenial(SMTPViolationError):
     pass
 
 
@@ -84,6 +63,8 @@ class SMTPSession(object):
         self._authenticator = authenticator
         
         self._command_arguments = None
+        self._close_connection_after_response = False
+        self._is_connected = True
         self._message = None
         
         self.hostname_regex = re.compile(regex_string, re.IGNORECASE)
@@ -208,7 +189,7 @@ class SMTPSession(object):
             print base_msg % (smtp_command, name_handler_method)
             self.reply(451, 'Temporary Local Problem: Please come back later')
         else:
-            # Don't catch InvalidParametersException here - else the state would
+            # Don't catch InvalidParametersError here - else the state would
             # be moved forward. Instead the handle_input will catch it and send
             # out the appropriate reply.
             handler_method()
@@ -216,19 +197,39 @@ class SMTPSession(object):
     def _evaluate_decision(self, decision):
         return (decision in [True, None])
     
+    def _is_multiline_reply(self, reply_message):
+        return (not isinstance(reply_message, basestring))
+    
+    def _send_custom_response(self, reply):
+        code, custom_response = reply
+        if self._is_multiline_reply(custom_response):
+            self.multiline_reply(code, custom_response)
+        else:
+            self.reply(code, custom_response)
+    
+    def _evaluate_policydecision_result(self, result):
+        decision = self._evaluate_decision(result.is_command_acceptable())
+        response_sent = result.use_custom_reply()
+        if result.close_connection_before_response():
+            self.close_connection()
+            response_sent = True
+        if result.use_custom_reply():
+            self._send_custom_response(result.get_custom_reply())
+        if result.close_connection_after_response():
+            self.please_close_connection_after_response()
+        return decision, response_sent
+    
     def is_allowed(self, acl_name, *args):
-        if self._policy != None:
+        if self._policy is not None:
             decider = getattr(self._policy, acl_name)
             result = decider(*args)
             if result in [True, False, None]:
                 return self._evaluate_decision(result), False
+            elif hasattr(result, 'is_command_acceptable'):
+                return self._evaluate_policydecision_result(result)
             elif len(result) == 2:
                 decision = self._evaluate_decision(result[0])
-                code, custom_response = result[1]
-                if not isinstance(custom_response, basestring):
-                    self.reply(code, custom_response)
-                else:
-                    self.multiline_reply(code, custom_response)
+                self._send_custom_response(result[1])
                 return decision, True
             raise ValueError('Unknown policy response')
         return True, False
@@ -253,33 +254,37 @@ class SMTPSession(object):
                 self.reply(554, 'SMTP service not available')
             self.close_connection()
     
-    
     def handle_input(self, smtp_command, data=None):
         """Processes the given SMTP command with the (optional data).
         [PUBLIC API]
         """
         self._command_arguments = data
+        self.please_close_connection_after_response(False)
         command = smtp_command.upper()
         try:
-            # SMTP commands must be treated as case-insensitive
-            self.state.execute(self, command)
-        except StateMachineError:
-            if command not in self.valid_commands:
-                self.reply(500, 'unrecognized command "%s"' % smtp_command)
-            else:
-                msg = 'Command "%s" is not allowed here' % smtp_command
-                allowed_transitions = self.state.transitions(self)
-                if len(allowed_transitions) > 0:
-                    msg += ', expected on of %s' % allowed_transitions
-                self.reply(503, msg)
-        except InvalidParametersException, e:
-            if not e.response_sent:
-                msg = 'Syntactically invalid %s argument(s)' % smtp_command
-                self.reply(501, msg)
-        except PolicyDenial, e:
-            if not e.response_sent:
-                self.reply(e.code, e.reply_text)
-        self._command_arguments = None
+            try:
+                # SMTP commands must be treated as case-insensitive
+                self.state.execute(self, command)
+            except StateMachineError:
+                if command not in self.valid_commands:
+                    self.reply(500, 'unrecognized command "%s"' % smtp_command)
+                else:
+                    msg = 'Command "%s" is not allowed here' % smtp_command
+                    allowed_transitions = self.state.transitions(self)
+                    if len(allowed_transitions) > 0:
+                        msg += ', expected on of %s' % allowed_transitions
+                        self.reply(503, msg)
+            except InvalidParametersError, e:
+                if not e.response_sent:
+                    msg = 'Syntactically invalid %s argument(s)' % smtp_command
+                    self.reply(501, msg)
+            except PolicyDenial, e:
+                if not e.response_sent:
+                    self.reply(e.code, e.reply_text)
+        finally:
+            if self.should_close_connection_after_response():
+                self.close_connection()
+            self._command_arguments = None
     
     def input_exceeds_limits(self):
         """Called when the client sent a message that exceeded the maximum 
@@ -289,19 +294,27 @@ class SMTPSession(object):
     def reply(self, code, text):
         """This method returns a message to the client (actually the session 
         object is responsible of actually pushing the bits)."""
-        #print 'code, text', code, text
         self._command_parser.push(code, text)
     
-    
     def multiline_reply(self, code, responses):
         """This method returns a message with multiple lines to the client 
         (actually the session object is responsible of actually pushing the 
         bits)."""
         self._command_parser.multiline_push(code, responses)
     
+    def please_close_connection_after_response(self, value=None):
+        if value is None:
+            value = True
+        self._close_connection_after_response = value
+    
+    def should_close_connection_after_response(self):
+        return self._close_connection_after_response
+    
     def close_connection(self):
         "Request a connection close from the SMTP session handling instance."
-        self._command_parser.close_when_done()
+        if self._is_connected:
+            self._is_connected = False
+            self._command_parser.close_when_done()
     
     
     # -------------------------------------------------------------------------
@@ -339,7 +352,7 @@ class SMTPSession(object):
         helo_string = (self._command_arguments or '').strip()
         valid_hostname_syntax = (self.hostname_regex.match(helo_string) != None)
         if not valid_hostname_syntax:
-            raise InvalidParametersException(helo_string)
+            raise InvalidParametersError(helo_string)
         else:
             decision, response_sent = self.is_allowed(policy_methodname, helo_string, self._message)
             if decision:
@@ -367,7 +380,7 @@ class SMTPSession(object):
         assert response_sent == False
         if self._authenticator == None:
             self.reply(535, 'AUTH not available')
-            raise InvalidParametersException(response_sent=True)
+            raise InvalidParametersError(response_sent=True)
         credentials_correct = \
             self._authenticator.authenticate(username, password, self._message.peer)
         if credentials_correct:
@@ -381,14 +394,14 @@ class SMTPSession(object):
         try:
             credentials = base64.decodestring(base64_credentials)
         except binascii.Error:
-            raise InvalidParametersException(base64_credentials)
+            raise InvalidParametersError(base64_credentials)
         else:
-            match = re.search('^\x00([^\x00]*)\x00([^\x00]*)$', credentials)
+            match = re.search('^[^\x00]*\x00([^\x00]*)\x00([^\x00]*)$', credentials)
             if match:
                 username, password = match.group(1), match.group(2)
                 self._check_password(username, password)
             else:
-                raise InvalidParametersException(credentials)
+                raise InvalidParametersError(credentials)
     
     def _split_mail_from_parameter(self, data):
         sender = data
@@ -417,7 +430,7 @@ class SMTPSession(object):
             if max_message_size != None:
                 if announced_size > max_message_size:
                     self.reply(552, 'message exceeds fixed maximum message size')
-                    raise InvalidParametersException('MAIL FROM', response_sent=True)
+                    raise InvalidParametersError('MAIL FROM', response_sent=True)
     
     def smtp_mail_from(self):
         data = self._command_arguments
@@ -429,7 +442,7 @@ class SMTPSession(object):
             self._check_mail_extensions(extensions)
         elif len(extensions) > 0:
             self.reply(501, 'No SMTP extensions allowed for plain SMTP')
-            raise InvalidParametersException('MAIL', response_sent=True)
+            raise InvalidParametersError('MAIL', response_sent=True)
         decision, response_sent = self.is_allowed('accept_from', sender, self._message)
         if decision:
             self._message.smtp_from = sender
@@ -442,7 +455,7 @@ class SMTPSession(object):
         match = re.search('^<?(.*?)>?$', parameter)
         if match:
             return match.group(1)
-        raise InvalidParametersException(parameter)
+        raise InvalidParametersError(parameter)
     
     def smtp_rcpt_to(self):
         # TODO: Check for good email address!
