@@ -30,13 +30,17 @@ tests using pymta:
 # THE SOFTWARE.
 
 from Queue import Queue
+import random
+import socket
 import threading
+import time
+from unittest import TestCase
 
-from pymta.api import IMessageDeliverer
+from pymta.api import IMessageDeliverer, IMTAPolicy
 from pymta.mta import PythonMTA
 
 
-__all__ = ['BlackholeDeliverer', 'DebuggingMTA', 'MTAThread']
+__all__ = ['BlackholeDeliverer', 'DebuggingMTA', 'MTAThread', 'SMTPTestCase']
 
 
 class BlackholeDeliverer(IMessageDeliverer):
@@ -94,3 +98,66 @@ class MTAThread(threading.Thread):
                       "termination!"
 
 
+class SMTPTestCase(TestCase):
+    """The SMTPTestCase is a unittest.TestCase and provides you with a running
+    MTA listening on 'localhost:[8000-40000]' which you can use in your 
+    tests. No messages will be delivered to the outside world because the MTA
+    configured by default uses the BlackholeDeliverer.
+    
+    Please make sure that you call the super method for setUp and tearDown."""
+    
+    def setUp(self):
+        super(SMTPTestCase, self).setUp()
+        self.hostname = 'localhost'
+        self.listen_port = random.randint(8000, 40000)
+        self.mta_thread = None
+        self.init_mta()
+    
+    def build_mta(self, hostname, listen_port, deliverer, policy_class=None):
+        """Return a PythonMTA instance which is configured according to your
+        needs."""
+        return DebuggingMTA(hostname, listen_port, deliverer,
+                            policy_class=policy_class)
+    
+    def init_mta(self, policy_class=IMTAPolicy):
+        """Starts the MTA in a separate thread with a BlackholeDeliver. This
+        method also ensures that the MTA is really listening on the specified
+        port."""
+        self.stop_mta()
+        self.deliverer = BlackholeDeliverer
+        self.mta = self.build_mta(self.hostname, self.listen_port, 
+                                  self.deliverer, policy_class)
+        self.mta_thread = MTAThread(self.mta)
+        self.mta_thread.start()
+        
+        self._try_to_connect_to_mta(self.hostname, self.listen_port)
+    
+    def _try_to_connect_to_mta(self, host, port):
+        tries = 0
+        while tries < 10:
+            try:
+                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+                sock.connect((host, port))
+            except socket.error:
+                tries += 1
+                time.sleep(0.1)
+            else:
+                return
+        assert False, 'MTA not reachable'
+    
+    def stop_mta(self):
+        if self.mta_thread is not None:
+            self.mta_thread.stop()
+            self.mta_thread = None
+    
+    def tearDown(self):
+        """Stops the MTA thread."""
+        self.stop_mta()
+        super(SMTPTestCase, self).tearDown()
+    
+    def get_received_messages(self):
+        """Return a list of received messages which are stored in the 
+        BlackholeDeliverer."""
+        return self.deliverer.received_messages
+
+
