@@ -24,10 +24,9 @@
 
 import smtplib
 import socket
-from unittest import TestCase
 
 from pymta.api import IMTAPolicy
-from pymta.test_util import BlackholeDeliverer, DebuggingMTA, MTAThread
+from pymta.test_util import DebuggingMTA, SMTPTestCase
 
 from tests.util import DummyAuthenticator
 
@@ -35,45 +34,35 @@ from tests.util import DummyAuthenticator
 rfc822_msg = 'Subject: Test\n\nJust testing...'
 
 
-class BasicSMTPTest(TestCase):
-    """This test uses the SMTP protocol to check the whole server."""
 
-    def setUp(self):
-        self.hostname = 'localhost'
-        self.listen_port = 8025
-        self.init_mta()
+class BasicSMTPTest(SMTPTestCase):
+    """This test uses the SMTP protocol to check the whole server."""
+    
+    def build_mta(self, hostname, listen_port, deliverer, policy_class=None):
+        return DebuggingMTA(hostname, listen_port, deliverer,
+                            authenticator_class=DummyAuthenticator,
+                            policy_class=policy_class)
     
     def init_mta(self, policy_class=IMTAPolicy):
-        self.deliverer = BlackholeDeliverer
-        self.mta = DebuggingMTA(self.hostname, self.listen_port, self.deliverer,
-                                policy_class=policy_class, 
-                                authenticator_class=DummyAuthenticator)
-        self.mta_thread = MTAThread(self.mta)
-        self.mta_thread.start()
-        
+        super(BasicSMTPTest, self).init_mta(policy_class)
         self.connection = smtplib.SMTP()
         self.connection.set_debuglevel(0)
         self.connection.connect(self.hostname, self.listen_port)
     
     def stop_mta(self):
-        try:
-            self.connection.quit()
-        except smtplib.SMTPServerDisconnected:
-            pass
-        self.mta_thread.stop()
-    
-    def tearDown(self):
-        self.stop_mta()
+        if getattr(self, 'connection', None) is not None:
+            try:
+                self.connection.quit()
+            except smtplib.SMTPServerDisconnected:
+                pass
+        super(BasicSMTPTest, self).stop_mta()
     
     def test_helo(self):
         code, replytext = self.connection.helo('foo')
         self.assertEqual(250, code)
     
-    def _get_received_messages(self):
-        return self.deliverer.received_messages
-    
     def _check_received_mail(self, expected_recipients, expected_helo=None):
-        queue = self._get_received_messages()
+        queue = self.get_received_messages()
         self.assertEqual(1, queue.qsize())
         msg = queue.get()
         if expected_helo != None:
@@ -127,7 +116,7 @@ class BasicSMTPTest(TestCase):
         self.connection.sendmail('x@example.com', 'foo@example.com', rfc822_msg)
         self.connection.sendmail('x@example.com', 'bar@example.com', rfc822_msg)
         
-        queue = self._get_received_messages()
+        queue = self.get_received_messages()
         self.assertEqual(2, queue.qsize())
         first_msg = queue.get()
         self.assertNotEqual(None, first_msg.smtp_helo)
@@ -144,7 +133,7 @@ class BasicSMTPTest(TestCase):
         self.connection.sendmail('from@example.com', 'foo@example.com', msg)
         self.connection.quit()
         
-        queue = self._get_received_messages()
+        queue = self.get_received_messages()
         self.assertEqual(1, queue.qsize())
         received_msg = queue.get()
         self.assertEqual(msg, received_msg.msg_data)
