@@ -41,9 +41,11 @@ class MockCommandParser(object):
         pass
     
     def push(self, code, text):
+        assert self.open
         self.replies.append((code, text))
     
     def multiline_push(self, code, lines):
+        assert self.open
         self.replies.append((code, lines))
     
     def close_when_done(self):
@@ -81,7 +83,7 @@ class CommandParserTestCase(TestCase):
                                    deliverer=self.deliverer,
                                    policy=policy, authenticator=authenticator)
         self.session.new_connection('127.0.0.1', 4567)
-        
+    
     def check_reply_code(self, code, reply_text, expected_first_digit):
         first_code_digit = int(str(code)[0])
         smtp_reply = "%s %s" % (code, reply_text)
@@ -99,7 +101,7 @@ class CommandParserTestCase(TestCase):
     def close_connection(self):
         self.send('quit')
         code, reply_text = self.command_parser.replies[-1]
-        self.assertTrue(221, code)
+        self.assertEqual(221, code)
         self.assertEqual('localhost closing connection', reply_text)
         self.assertEqual(False, self.command_parser.open)
 
